%% setup parameters NInA
clear;
input_image = "D:\data\GRABDA and ACh\tif\GRABDA\test\10Hz_2020_03_14_350 slice1_b.tif"; 

output_result = "D:\data\GRABDA and ACh\tif\GRABDA\test\test.mat"; 

% basic infor
sampling_frequency = 10; 
    % sampling frequency in Hz (frame/s)
frame_max = 800;
    % maximum frames to process. The analysis will take a lot of RAM

blur_factor_xy = 2; 
blur_round_xy = 1;         
blur_factor_z = 1;
blur_round_z = 1;

% pixel parameters
threshold_ratio = 0.005;  % minimum ratio of increase for event detection dF/F0
    switch_threshold_ratio_sd = 1; 
        threshold_ratio_sd_fold = 1;
    ratio_duration_min = 0.9;  
    ratio_duration_max = 5;  
    ratio_max =  4; 
    ratio_min = -0.2;
    
threshold_ratio_dt2 = 0.1;     % minimum speed of increase in df/f*frequency squared 
threshold_ratio_dt2_SD_fold = 1;  

    threshold_ratio_dt2_detection_range = -1 : 1;    % frame range for searching ratio_dt2, It has to be within z_remove  
    z_startpoint_remove = 50;   % number need to be at least 3, frame to start the analysis, this will avoid the first few slices for analysis. This will only influence the analysis but not the images, not influenced  by switch_ROI_quantification
    z_endpoint_remove = 50; % frames to remove for the analysis, this will remove the last few slices. This only influences the analysis not influenced  by switch_ROI_quantification

      
% event parameters
event_size_min = 2000;  % minimum voxel number of an event
event_size_max = 100000000; % maximum voxel number of an event
event_duration_min = 1.2; % minimum duration of an event
event_duration_max = 6; % maximum duration of an event
event_pixelperlayer_min = 20; % minimum pixel number in each layer/page/timepoint
event_maxlayer_min = 500; % minimum pixel in the biggest layer
event_pixelperlayer_max = 100000000; % maximum pixel number in each layer/page/timepoint
iteration_round_max = 4; % maximum iteration for event seperation
event_segregation_epsilon_factor = 0.25; 
    % threshold for event segmentaion, meaning the ratio of diameter of the event as a threshold for clustering of center of mass in each layer
event_segregation_duration_min = 1;  % minumum duration in second of neighbers to be considered as a cluster during segmentation, basically an event need to have at least * layers * samplefrequency to be able to cut an event.
event_result_align_length = 1000;  
    % define the length of array in pixel to contain the aligned events

switch_dynamic_F0 = 1; % 1 indicate F0 is based on dynamic intensity of each frame after z direction smooth, otherwise the code use frame defined as background
    zsmooth_duration = 10; % in s, indicate the window of mean average in z direction for estimation of basal level(s). Better be an odd number when multiplied by samplefrequency
    zsmooth_round = 10; % indicate the times of iteration for estimation of basal level.
switch_bgsubtraction = 1; % 0 inidcate there is no need to subtract background, 1 will initiate ROI subtraction
    background_min = 250; % add a value after background subtraction if bacground is smaller than this number to avoid division by zero     
    switch_lowbackground_quantification = 0; % 1 means dots with mini background is also quantified
    switch_ROI_quantification = 1; % only quantify events within this ROI, This will not influence the generation of pixel based analysis
    


%% define ROI
    disp('running'); % indicate progress number in the plot '9. one file finished';    
    disp(input_image);
    image = fun_readimage(input_image, 1);  
    imshow(image,[min(image,[],'all'),max(image,[],'all')]); 
    pause(0.5); 
    title('define background', 'color','b');     
    [background] = fun_define_ROI();
    pause(0.5); 
    title('define ROI for quantification', 'color', 'r'); 
    [ROI_quantification] = fun_define_ROI();
    pause(0.5); 
    close('all');
%% Run anlaysis

    %% loading file
    tic;   
    image = fun_readimage(input_image, frame_max);  % read the file stack     
    frame_n = size(image, 3)- z_startpoint_remove - z_endpoint_remove;     
    recordingtime = frame_n/sampling_frequency; % calculate duration of recording in the unit of s  
    disp '0. image loaded';
    toc;  

    %% generate bgsubtracted image based on selected ROI
    if switch_bgsubtraction == 1                   
        [image_bgsubtracted] = fun_background_subtraction(image, background, background_min); % generate background-subtracted image     
    else
        image_bgsubtracted = image;
    end  
    
 
    %% generate blured image
    tic;    
    b_image_bgsubtracted = fun_imageblur_xy(image_bgsubtracted, blur_factor_xy, blur_round_xy); 
    if blur_round_z > 1
        for z = 1 : blur_round_z
            b_image_bgsubtracted = movmean(b_image_bgsubtracted, blur_factor_z, 3);
        end
    end
    
    if switch_lowbackground_quantification == 1
        b_binary_F0 = b_image_bgsubtracted > 0; 
    else
        b_binary_F0 = b_image_bgsubtracted >  background_min;
    end
    if switch_dynamic_F0 == 1
        [b_image_F0] = fun_zsmooth(b_image_bgsubtracted, zsmooth_duration, sampling_frequency, zsmooth_round); % estimate basal fluorescence  
    else        
        b_image_F0 = mean(b_image_bgsubtracted, 3);
    end
    b_image_dF = b_image_bgsubtracted - b_image_F0; % calculate df
    b_image_ratio = b_image_dF ./ b_image_F0 .* b_binary_F0; % generate ratio and ignore areas with low background unit (df/f/s) 
    b_image_ratio(b_image_ratio > ratio_max) = ratio_max; % cut extreme number
    b_image_ratio(b_image_ratio < ratio_min) = ratio_min; % cut extreme number   
   
    disp '1. blured image_ratio generated';  
    toc;
   
    %% analyze map_event
    if switch_threshold_ratio_sd == 1 
        b_image_ratio_sd = std(b_image_ratio(:,:,1+ z_startpoint_remove: size(b_image_ratio,3) - z_endpoint_remove),1,3);
        threshold_ratio_sd = b_image_ratio_sd .* threshold_ratio_sd_fold;
        threshold_ratio_sd(threshold_ratio_sd<threshold_ratio) = threshold_ratio;
        threshold_ratio_used = threshold_ratio_sd;
    else
        threshold_ratio_used = threshold_ratio;
    end
    
    
    [map_event, map_event_count] = fun_event_detection_pixel...
        (b_image_ratio, b_binary_F0, threshold_ratio_used, threshold_ratio_dt2,...
            threshold_ratio_dt2_SD_fold, threshold_ratio_dt2_detection_range, ...
                ratio_duration_min, ratio_duration_max,...
                sampling_frequency, z_startpoint_remove, z_endpoint_remove);    
    disp '2. eventmap generated';
    toc;   
    
       %% analysis based on pixels along z direction     
    binary_F0 = image_bgsubtracted > background_min; % only consider areas with high enough background
    if switch_dynamic_F0 == 1
        [image_F0] = fun_zsmooth(image_bgsubtracted, zsmooth_duration, sampling_frequency, zsmooth_round); % estimate basal fluorescence  
    else
        image_F0 = mean(image_bgsubtracted, 3);        
    end        
    image_dF = image_bgsubtracted - image_F0; % calculate df
    image_ratio = image_dF ./ image_F0 .* binary_F0; % generate ratio and ignore areas with low background unit (df/f/s)
    image_ratio(image_ratio > ratio_max) = ratio_max; % cut extreme number
    image_ratio(image_ratio < ratio_min) = ratio_min; % cut extreme number
    map_event_ratio = image_ratio .* map_event;     
    
    temp_ROI_ratio_overall = mean(image_dF.*ROI_quantification, 1:2)./mean(image_bgsubtracted.*ROI_quantification, 1:2);
    ROI_ratio_overall = [];
    ROI_ratio_overall(:,1) = temp_ROI_ratio_overall(1,1,:);  % generate the mean ratio within the whole ROI
    i = 1;
    result.ROI_quantification.ROI_ratio_overall(1,i) = {ROI_ratio_overall};
    
    [ROI_area, ROI_areabyratio, ROI_ratio] = fun_quantify_ROI(map_event_ratio); % quantify ROI and save to result    
    ROI_area = ROI_area ;
    ROI_areabyratio = ROI_areabyratio ;
    ROI_ratio(ROI_ratio>ratio_max) = ratio_max;
    ROI_ratio(ROI_ratio < ratio_min) = ratio_min;
    result.ROI_quantification.ROI_area(1,i) = {ROI_area};
    result.ROI_quantification.ROI_areabyratio(1,i) = {ROI_areabyratio};
    result.ROI_quantification.ROI_ratio(1,i) = {ROI_ratio};
    result.ROI_quantification.ROI_area_max(1,i) = max(ROI_area);
    result.ROI_quantification.ROI_areabyratio_max(1,i) = max(ROI_areabyratio);
    result.ROI_quantification.ROI_ratio_max(1,i) = max(ROI_ratio);     
    disp '3. image_ratio generated';    
    toc;
     
  
    %% analysis based on events  
    tic;
    if switch_ROI_quantification == 1  % Remove other areas if ROI for quantification is required
        map_event = map_event .* ROI_quantification;
    end
    [list_event, list_event_single] = fun_event_detection(map_event, ...  % generate event list containing all voxel infor
        event_size_min, event_size_max, event_duration_min, event_duration_max,...
            event_pixelperlayer_min, event_pixelperlayer_max, event_maxlayer_min,...
                event_segregation_epsilon_factor, event_segregation_duration_min, sampling_frequency, iteration_round_max);     
            
    [map_event_new] = fun_generate_new_map_event(map_event, list_event); % generate new map_event based on event
    [map_event_single] = fun_generate_new_map_event(map_event, list_event_single); % generate map_event based on single event  
    clear map_event    
    map_event_ratio_single = map_event_ratio .* map_event_single; % generate new map_event single ratio based on single event
    map_event_ratio = map_event_ratio .* map_event_new;  % generate new map_event ratio based on event     
    disp '4. new map_event and map_event_ratio generated';
    toc;
  
    
    %% event quantification
    tic;
    [area, areabyratio, ratio, volume, halfwidth_duration_area] = fun_event_quantification(map_event_ratio, list_event);
    [area_single, areabyratio_single, ratio_single, volume_single, halfwidth_duration_area_single] = fun_event_quantification(map_event_ratio, list_event_single);       
   
    volume = volume./sampling_frequency; % such that the volume unit becomes um^2 * s
    halfwidth_duration_area = halfwidth_duration_area./sampling_frequency; % convert halfwidth to seconds
   
    volume_single = volume_single/sampling_frequency; % such that the volume unit becomes um^2 * s
    halfwidth_duration_area_single = halfwidth_duration_area_single./sampling_frequency; % convert halfwidth to seconds
    clear map_event_ratio
    
    [area_aligned] = fun_aligntopeak(area, event_result_align_length); % generate event curve aligned to the peak
    [areabyratio_aligned] = fun_aligntopeak(areabyratio, event_result_align_length);% generate averaged areabyratio aligned to the peak
    [ratio_aligned] = fun_aligntopeak(ratio, event_result_align_length); % generate averaged ratio aligned to the peak
    [area_aligned_single] = fun_aligntopeak(area_single, event_result_align_length);  
    [areabyratio_aligned_single] = fun_aligntopeak(areabyratio_single, event_result_align_length);
    [ratio_aligned_single] = fun_aligntopeak(ratio_single, event_result_align_length);    
    
    disp '5. event quantification finished';
    toc;
 
    %% generate resultfile 
    tic;    
    
    if ~isempty(area)
        result.allevent.area(1, i) = {area};  % timecourse of influenced area
        result.allevent.areabyratio(1, i) = {areabyratio}; % timecourse of areabyratio
        result.allevent.ratio(1, i) = {ratio};  % timecourse of ratio
        ratio_dt2_max = max((ratio(2:end,:)-ratio(1:end-1,:))*sampling_frequency, [], 1);
        result.allevent.ratio_dt2_max(1, i) = {ratio_dt2_max};  % max ratio change
        result.allevent.area_algined(1, i) = {area_aligned}; % timecourse of area aligned to the peak
        result.allevent.areabyratio_aligned(1, i) = {areabyratio_aligned}; % timecourse of areabyratio aligned to the peak
        result.allevent.ratio_aligned(1, i) = {ratio_aligned};  % timecourse of ratio aligned to the peak    
        result.allevent.max_area(1, i) = {max(area, [], 1)}; % maximum area influenced
        result.allevent.max_areabyratio(1, i) = {max(areabyratio, [], 1)}; % max areabyratio influenced
        result.allevent.max_ratio(1, i) = {max(ratio, [], 1)}; % max ratio
        result.allevent.volume(1, i) = {volume}; % volume of each event
        result.allevent.halfwidth_duration_area(1, i) = {halfwidth_duration_area}; % halfwidth of each event calculated by are of influence
        result.allevent.area_average(1:size(area_aligned,1), i) = mean(area_aligned, 2); % averaged alinged area of influence
        result.allevent.areabyratio_average(1:size(areabyratio_aligned,1), i) = mean(areabyratio_aligned, 2); % averaged areabyratio 
        result.allevent.ratio_average(1:size(ratio_aligned,1), i) = mean(ratio_aligned, 2); % averaged ratio
        result.allevent.max_area_average(1, i) = mean(max(area, [], 1)); % averaged max area
        result.allevent.max_areabyratio_average(1, i) = mean(max(areabyratio, [], 1)); % averaged max areabyratio
        result.allevent.max_ratio_average(1, i) = mean(max(ratio, [], 1));  % mean max ratio   
        result.allevent.volume_average(1, i) = mean(volume, 2); % mean volume
        result.allevent.halfwidth_duration_area_average(1, i) = mean(halfwidth_duration_area, 2); % averaged halfwidth duration
        result.allevent.frequency(1, i) = size(list_event, 1) ./ frame_n * sampling_frequency; % frequency
    else
        result.allevent.area(1, i) = {0};
        result.allevent.areabyratio(1, i) = {0};
        result.allevent.ratio(1, i) = {0}; 
        result.allevent.ratio_dt2_max(1, i) = {0};
        result.allevent.area_algined(1, i) = {0};
        result.allevent.areabyratio_aligned(1, i) = {0};
        result.allevent.ratio_aligned(1, i) = {0};      
        result.allevent.max_area(1, i) = {0};
        result.allevent.max_areabyratio(1, i) = {0};
        result.allevent.max_ratio(1, i) = {0};
        result.allevent.volume(1, i) = {0};
        result.allevent.halfwidth_duration_area(1, i) = {0}; % calculated by are of influence
        result.allevent.area_average(1:size(area_aligned,1), i) = 0;
        result.allevent.areabyratio_average(1:size(areabyratio_aligned,1), i) = 0;
        result.allevent.ratio_average(1:size(ratio_aligned,1), i) = 0; 
        result.allevent.max_area_average(1, i) = 0;
        result.allevent.max_areabyratio_average(1, i) = 0;
        result.allevent.max_ratio_average(1, i) = 0;     
        result.allevent.volume_average(1, i) = 0;
        result.allevent.halfwidth_duration_area_average(1, i) = 0;
        result.allevent.frequency(1, i) = 0;
    end
    
    if ~isempty(area_single)
        result.singleevent.area(1, i) = {area_single};
        result.singleevent.areabyratio(1, i) = {areabyratio_single};
        result.singleevent.ratio(1, i) = {ratio_single};
        ratio_dt2_max_single = max((ratio_single(2:end,:)-ratio_single(1:end-1,:))*sampling_frequency, [], 1);
        result.allevent.ratio_dt2_max(1, i) = {ratio_dt2_max_single};  % max ratio change
        result.singleevent.area_aligned(1, i) = {area_aligned_single};
        result.singleevent.areabyratio_aligned(1, i) = {areabyratio_aligned_single};
        result.singleevent.ratio_aligned(1, i) = {ratio_aligned_single};
        result.singleevent.max_area(1, i) = {max(area_single, [], 1)};
        result.singleevent.max_areabyratio(1, i) = {max(areabyratio_single, [], 1)};
        result.singleevent.max_ratio(1, i) = {max(ratio_single, [], 1)};    
        result.singleevent.volume(1, i) = {volume_single};
        result.singleevent.halfwidth_duration_area(1, i) = {halfwidth_duration_area_single};    
        result.singleevent.area_average(1:size(area_aligned_single,1), i) = mean(area_aligned_single, 2);
        result.singleevent.areabyratio_average(1:size(areabyratio_aligned_single,1), i) = mean(areabyratio_aligned_single, 2);
        result.singleevent.ratio_average(1:size(ratio_aligned_single,1), i) = mean(ratio_aligned_single, 2);
        result.singleevent.max_area_average(1, i) = mean(max(area_single, [], 1));
        result.singleevent.max_areabyratio_average(1, i) = mean(max(areabyratio_single, [], 1));
        result.singleevent.max_ratio_average(1, i) = mean(max(ratio_single, [], 1));    
        result.singleevent.volume_average(1, i) = mean(volume_single, 2);
        result.singleevent.halfwidth_duration_area_average(1, i) = mean(halfwidth_duration_area_single, 2);    
        result.singleevent.frequency(1, i) = size(list_event_single, 1) ./ frame_n * sampling_frequency;
    else
        result.singleevent.area(1, i) = {0};
        result.singleevent.areabyratio(1, i) = {0};
        result.singleevent.ratio(1, i) = {0};
        result.singleevent.ratio_aligned(1, i) = {0};
        result.singleevent.area_aligned(1, i) = {0};
        result.singleevent.areabyratio_aligned(1, i) = {0};
        result.singleevent.ratio_aligned(1, i) = {0};
        result.singleevent.max_area(1, i) = {0};
        result.singleevent.max_areabyratio(1, i) = {0};
        result.singleevent.max_ratio(1, i) = {0};    
        result.singleevent.volume(1, i) = {0};
        result.singleevent.halfwidth_duration_area(1, i) = {0};    
        result.singleevent.area_average(1:size(area_aligned_single,1), i) = 0;
        result.singleevent.areabyratio_average(1:size(areabyratio_aligned_single,1), i) = 0;
        result.singleevent.ratio_average(1:size(ratio_aligned_single,1), i) = 0;
        result.singleevent.max_area_average(1, i) = 0;
        result.singleevent.max_areabyratio_average(1, i) = 0;
        result.singleevent.max_ratio_average(1, i) = 0;    
        result.singleevent.volume_average(1, i) = 0;
        result.singleevent.halfwidth_duration_area_average(1, i) = 0;    
        result.singleevent.frequency(1, i) = 0;
    end
        
save(output_result, 'result');    
disp '6. result saved';
toc;

   
close('all');
clear;
toc;
disp 'All done';



