function [output_list_event, output_list_event_single] = fun_event_detection(input_map_event, ...
            input_event_size_min, input_event_size_max, input_event_duration_min, input_event_duration_max,...
                input_event_pixelperlayer_min, input_event_pixelperlayer_max, input_event_maxlayer_min,...
                    input_event_segregation_epsilon_factor, input_event_segregation_duration_min, input_samplingfrequency, input_iteration_round_max)

    input_event_segregation_number_min = ceil(input_event_segregation_duration_min .* input_samplingfrequency); % to make sure it is an integer
    iteration_round = 0;
    list_event_single = {};

    while (1) % at most iterate 5 times
        list_filter0 = {};
        list_filter1 = {};
        list_filter2 = {};        
        iteration_round = iteration_round + 1;        
        disp(strcat('iteration_round_', string(iteration_round)));
        %% generate list0 and volume0
        cc =  bwconncomp (input_map_event, 6);
        list0 = regionprops3(cc, 'VoxelList'); 
        volume0 = regionprops3(cc, 'Volume'); 
        if ~isempty(list0)
            n = 0;
            for i = 1 : size(list0, 1) % filter1 by size
                if volume0.Volume(i) >= input_event_size_min && volume0.Volume(i) <= input_event_size_max   
                    n = n + 1;      
                    list_filter0(n, 1) = list0.VoxelList(i);     
                end
            end
        end

        %% pixel per layer filter    
        if ~isempty(list_filter0)
            for i = 1 : size(list_filter0, 1) % filter by layer and correct raw images
                voxel = list_filter0{i}; % obtain voxel list for each object             
                page = voxel(:, 3); % obtain page info
                page_unique = unique(page); % obtain unique page number 
                max_n = 0;
                for j = 1 : size(page_unique, 1)
                    n = sum(page==page_unique(j)); % obtain the number of pixels in each layer
                    if n > max_n
                        max_n = n;
                    end
                    if n < input_event_pixelperlayer_min || n > input_event_pixelperlayer_max % convert the corresponding position to 0 in event ratio 
                        page_index = find(page==page_unique(j)); % find the index of this page
                        for k = 1 : size(page_index,1)
                            input_map_event (voxel(page_index(k), 2), voxel(page_index(k), 1), voxel(page_index(k), 3)) = 0; % change the corresponding position to 0, first column indicates x        
                        end
                    end                                  
                end
                if max_n < input_event_maxlayer_min % maxpixel needs to be bigger than minimum threshold
                    for k = 1 : size(voxel,1)
                       input_map_event (voxel(k, 2), voxel(k, 1), voxel(k, 3)) = 0; % change the corresponding position to 0, first column indicates x        
                    end
                end   
            end
        end                

        cc =  bwconncomp (input_map_event, 6); % one can control the connection pattern using this method, otherwise there is no way to control the connection mode
        list1 = regionprops3(cc, 'VoxelList'); 
        volume1 = regionprops3(cc, 'Volume'); 
        if ~isempty(list1)
            n = 0;   
            for i = 1 : size(list1, 1) % filter1 by size
                if volume1.Volume(i) >= input_event_size_min && volume1.Volume(i) <= input_event_size_max   
                    n = n + 1;      
                    list_filter1(n, 1) = list1.VoxelList(i);    % list_filter1 type is cell 
                end
            end    
        end 

        %% seperate complex events
         if ~isempty(list_filter1)             
             for i = 1 : size(list_filter1, 1)
                 mass_center = [];
                 page_info = [];
                 voxel = list_filter1{i}; % obtain voxel list for each object           
                 page = voxel(:, 3); % page info
                 page_unique = unique(page);  % find unique pages  
                 page_n = length(page_unique); % number of layers
                 for j = 1 : size(page_unique, 1) % find the mass center for each layer
                    page_index = find(page==page_unique(j)); % find the index of this page                    
                    mass_center(j, 1) = mean(voxel(page_index, 1));
                    mass_center(j, 2) = mean(voxel(page_index, 2));
                    page_info (j, 1) = mean(voxel(page_index, 3)); % every page_index should be the same
                 end                 
                 

                 epsilon = sqrt(size(voxel, 1)/page_n) * input_event_segregation_epsilon_factor;  % calculate threshold for dbscan percentage of diameter of biggest area
                 cluster = dbscan(mass_center, epsilon, input_event_segregation_number_min); %generate cluster index for each layer                 
                 if max(cluster) < 2 && iteration_round == 1  % generate single event list
                      list_event_single(size(list_event_single, 1)+1,1) = {voxel};
                 end                
                 if max(cluster) >= 2 % only seperate them when there is more groups                  
                    for c = 1 : max(cluster) - 1 % cut for each cluster but not the last one, c indicate cluster #  
                        cutting_plane = zeros(size(input_map_event,1), size(input_map_event,2)); % an empty layer for cutting the raw map
                        cluster_index = find(cluster == c);
                        page_index = page_info(cluster_index);                        
                        for k = 1 : size(page_index, 1)
                            for m = 1 : size(voxel, 1)
                                 if voxel(m, 3) == page_index(k)
                                     cutting_plane(voxel(m, 2), voxel(m,1)) = 1;
                                 end
                            end
                        end  
                        
                        cc = bwconncomp(cutting_plane,8); % one have to use this method because regionprops does not consider 8 connections
                        list_cutting_plane = regionprops(cc, 'PixelList');                       
                        max_object_index = 1;
                        max_object_area = size(list_cutting_plane(1).PixelList, 1);
                        for y = 2: size(list_cutting_plane, 1) % find the maximum object so that one can remove other noisy dots
                            if size(list_cutting_plane(y).PixelList, 1) > max_object_area
                                max_object_area = size(list_cutting_plane(y).PixelList, 1);
                                max_object_index = y;
                            end
                        end
                        max_object_pixellist = list_cutting_plane(max_object_index).PixelList;
                        cutting_plane = zeros(size(input_map_event,1), size(input_map_event,2)); % generat new cutting plane
                        for u = 1 : size(max_object_pixellist, 1)
                            cutting_plane(max_object_pixellist(u, 2), max_object_pixellist(u, 1)) = 1;
                        end                        
                        cutting_plane_edge = edge(cutting_plane, 'Canny');  % find the edge of the cutting plane
                        cutting_plane_edge = cutting_plane_edge == 0;
                        input_map_event(:,:,min(page_index) : max(voxel(:,3))) =...  % cut through the corresponding pages
                            input_map_event(:,:,min(page_index) : max(voxel(:,3))) .* cutting_plane_edge;                   
                    end                  
                 end                            
             end         
         end

        cc =  bwconncomp (input_map_event, 6);
        list2 = regionprops3(cc, 'VoxelList'); 
        volume2 = regionprops3(cc, 'Volume'); 
        if ~isempty(list2)
            n = 0;
            for i = 1 : size(list2, 1) % filter1 by size
                if volume2.Volume(i) >= input_event_size_min && volume2.Volume(i) <= input_event_size_max   
                    n = n + 1;      
                    list_filter2(n, 1) = list2.VoxelList(i);     
                end
            end   
        end
        %% check the result and determine whether to iterate
        if (size(list_filter0, 1) == size(list_filter1, 1)...
                && size(list_filter0, 1) == size(list_filter2, 1)...
                && size(list_filter1, 1) == size(list_filter2, 1))
            break;
        end
        if iteration_round >= input_iteration_round_max
            break;
        end
    end
    
    %% filter by duration
    frame_n_min = input_event_duration_min * input_samplingfrequency;
    frame_n_max = input_event_duration_max * input_samplingfrequency;
    list_filter3 = {};
    list_event_single_filter3 = {};
    if ~isempty(list_filter2)  % duration filter for all events
        n = 0;
        for i = 1 : size(list_filter2, 1)
            voxel = cell2mat(list_filter2(i , 1));
            page_n = length(unique(voxel(:,3)));
            if page_n >= frame_n_min && page_n <= frame_n_max
                n = n + 1;
                list_filter3(n,1) = list_filter2(i, 1);
            end
        end
    end
    if ~isempty(list_event_single)  % duration filter for all events
        n = 0;
        for i = 1 : size(list_event_single, 1)
            voxel = cell2mat(list_event_single(i, 1));
            page_n = length(unique(voxel(:,3)));
            if page_n >= frame_n_min && page_n <= frame_n_max
                n = n + 1;
                list_event_single_filter3(n,1) = list_event_single(i, 1);
            end
        end
    end
     
    output_list_event = list_filter3;
    output_list_event_single = list_event_single_filter3;

end
  
