function [output_eventmap, output_frequencymap] = fun_event_detection_pixel...
        (input_ratio, input_binary_F0, input_ratio_threshold, input_ratio_dt2_threshold,...
            input_threshold_ratio_dt2_SD_fold, input_threshold_ratio_dt2_detection_range,... 
                input_ratio_duration_min, input_ratio_duration_max,...
                    input_sampling_frequency, input_z_startpoint_remove, input_z_endpoint_remove)
        
    [ratio_blur] =  input_ratio .* input_binary_F0; % generate blurred ratio, change the ratio of low-background area to 0
    
    clear input_ratio input_binary_F0;
    
    ratio_blur_dt2(:,:,2:size(ratio_blur,3)) = ...
        (ratio_blur(:,:,2:end) - ratio_blur(:,:,1:end-1)).* input_sampling_frequency;  % generate the square of ratio dt (Ft - Ft-1)^2 the first frame is 0
        
    binary_ratio_blur = (ratio_blur > input_ratio_threshold) .* 1 ; % generate initial binary map and convert to double
    ratio_blur_max = max(ratio_blur, [], 3);  % show the results as a guide for thresholding
    ratio_blur_max_nonzero = ratio_blur_max (ratio_blur_max > 0);
    clear ratio_blur
    
    binary_ratio_blur_dt2 = (ratio_blur_dt2 > input_ratio_dt2_threshold) .* 1 ;  % generate dt2 map for acceleration   
    ratio_blur_dt2_SD = std(ratio_blur_dt2, 1, 3); % 1 means using N instead of n-1 to calculate std, 3 menas z direction
    ratio_blur_dt2_SD_threshold = ratio_blur_dt2_SD .* input_threshold_ratio_dt2_SD_fold;
    binary_ratio_blur_dt2_SD = (ratio_blur_dt2 > ratio_blur_dt2_SD_threshold) .* 1 ;
    binary_ratio_blur_dt2 = binary_ratio_blur_dt2 .* binary_ratio_blur_dt2_SD;    
    ratio_blur_dt2_max = max(ratio_blur_dt2, [], 3);
    ratio_blur_dt2_max_nonzero = ratio_blur_dt2_max(ratio_blur_dt2_max > 0); 
    clear  ratio_blur_dt2  % to save RAM
    
    ratio_histogram = figure; 
    subplot(2,1,1);
    histogram(ratio_blur_max_nonzero);
    xlim([-std(ratio_blur_max_nonzero, [], 'all') 5*std(ratio_blur_max_nonzero, [], 'all')]);  % show a range of 5 std
    title ('ratio blur max ( >0 )'); 
    subplot(2,1,2);
    histogram(ratio_blur_dt2_max_nonzero);
    xlim([-std(ratio_blur_dt2_max_nonzero, [], 'all') 5*std(ratio_blur_dt2_max_nonzero, [], 'all')]);  % show a range of 5 std
    title ('ratio blur dt2 ( >0 )');
    assignin('base', 'ratio_histogram', ratio_histogram); % move histogram plot to the base to save
    pause(1);   
    %% generate event map pixel by pixel
    
    output_frequencymap = zeros(size(binary_ratio_blur,1), size(binary_ratio_blur,2));
    input_ratio_window = floor(input_ratio_duration_min .* input_sampling_frequency);
    pattern = [0,ones(1, input_ratio_window)]; %generate pattern to search
    z_startpoint = input_z_startpoint_remove + 1; %define the startpoint for analysis
    z_endpoint = size(binary_ratio_blur, 3) - input_z_endpoint_remove; % define the endpoint for analysis
    frame_max = input_ratio_duration_max * input_sampling_frequency;
    
    for i = 1 : size(binary_ratio_blur, 1)  % refine the binary_ratio_blur by duration and whether the dt2 reaches threshold
        for j = 1 : size(binary_ratio_blur, 2)
            list = [];
            z(:) = binary_ratio_blur(i,j,:);
            zdt2(:) = binary_ratio_blur_dt2(i,j,:);
            list = strfind(z, pattern);
            if ~isempty(list)                
                for k = 1 : length(list)
                    sp = list(k) + 1;
                    for m = sp : length(z) % find the end of the event
                         if z(m) == 0
                              ep = m - 1;                                                  
                              break;
                         end
                    end  
                    if sp > z_startpoint && sp < z_endpoint && ep-sp < frame_max
                        if sum(zdt2(input_threshold_ratio_dt2_detection_range + sp)) >= 1   % only consider the defined range change to 2 if speed is fast enough 
                            binary_ratio_blur(i,j,sp:ep) = 2;
                            output_frequencymap(i,j) = output_frequencymap(i,j) + 1; %calculate events number
                        end
                    end
                end
            end        
        end  
    end     
    
    output_eventmap = (binary_ratio_blur==2) .* 1;             

end

